@@ -23,6 +23,8 @@ import xml.etree.ElementTree as ET
 import requests
 import six
 
+from scciclient.irmc import ipmi
+from scciclient.irmc import snmp
 
 DEBUG = False
 
@@ -56,6 +58,21 @@ class SCCIClientError(SCCIError):
         super(SCCIClientError, self).__init__(message)
 
 
+class SCCIRAIDNotReady(SCCIError):
+    """SCCIRAIDNotReady
+
+    This exception is used when a mechanism not applied
+    into a configuration on the iRMC yet
+    """
+    def __init__(self, message):
+        super(SCCIRAIDNotReady, self).__init__(message)
+
+
+class SCCISessionTimeout(SCCIError):
+    def __init__(self, message):
+        super(SCCISessionTimeout, self).__init__(message)
+
+
 """
 List of iRMC S4 supported SCCI commands
 
@@ -230,7 +247,7 @@ def get_share_type(share_type):
 
 
 def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
-             client_timeout=60, async=True, **kwargs):
+             client_timeout=60, do_async=True, **kwargs):
     """execute SCCI command
 
     This function calls SCCI server modules
@@ -241,7 +258,7 @@ def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
     :param port: port number of iRMC
     :param auth_method: irmc_username
     :param client_timeout: timeout for SCCI operations
-    :param async: async call if True, sync call otherwise
+    :param do_async: async call if True, sync call otherwise
     :returns: requests.Response from SCCI server
     :raises: SCCIInvalidInputError if port and/or auth_method params
              are invalid
@@ -263,19 +280,31 @@ def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
 
     try:
         header = {'Content-type': 'application/x-www-form-urlencoded'}
-        r = requests.post(protocol + '://' + host + '/config',
-                          data=cmd,
+        if kwargs.get('upgrade_type') == 'irmc':
+            with open(cmd, 'rb') as file:
+                data = file.read()
+            config_type = '/irmcupdate?flashSelect=255'
+        elif kwargs.get('upgrade_type') == 'bios':
+            with open(cmd, 'rb') as file:
+                data = file.read()
+            config_type = '/biosupdate'
+        else:
+            data = cmd
+            config_type = '/config'
+        r = requests.post(protocol + '://' + host + config_type,
+                          data=data,
                           headers=header,
                           verify=False,
                           timeout=client_timeout,
                           allow_redirects=False,
                           auth=auth_obj)
-        if not async:
+
+        if not do_async:
             time.sleep(5)
         if DEBUG:
             print(cmd)
             print(r.text)
-            print("async = %s" % async)
+            print("do_async = %s" % do_async)
         if r.status_code not in (200, 201):
             raise SCCIClientError(
                 ('HTTP PROTOCOL ERROR, STATUS CODE = %s' %
@@ -294,6 +323,9 @@ def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
         else:
             return r
 
+    except IOError as input_error:
+        raise SCCIClientError(input_error)
+
     except ET.ParseError as parse_error:
         raise SCCIClientError(parse_error)
 
@@ -486,3 +518,203 @@ def get_essential_properties(report, prop_keys):
     v['cpu_arch'] = 'x86_64'
 
     return {k: v[k] for k in prop_keys}
+
+
+def get_capabilities_properties(d_info,
+                                capa_keys,
+                                gpu_ids,
+                                fpga_ids=None,
+                                **kwargs):
+    """get capabilities properties
+
+    This function returns a dictionary which contains keys
+    and their values from the report.
+
+
+    :param d_info: the dictionary of ipmitool parameters for accessing a node.
+    :param capa_keys: a list of keys for additional capabilities properties.
+    :param gpu_ids: the list of string contains <vendorID>/<deviceID>
+    for GPU.
+    :param fpga_ids: the list of string contains <vendorID>/<deviceID>
+    for CPU FPGA.
+    :param kwargs: additional arguments passed to scciclient.
+    :returns: a dictionary which contains keys and their values.
+    """
+
+    snmp_client = snmp.SNMPClient(d_info['irmc_address'],
+                                  d_info['irmc_snmp_port'],
+                                  d_info['irmc_snmp_version'],
+                                  d_info['irmc_snmp_community'],
+                                  d_info['irmc_snmp_security'])
+    try:
+        v = {}
+        if 'rom_firmware_version' in capa_keys:
+            v['rom_firmware_version'] = \
+                snmp.get_bios_firmware_version(snmp_client)
+
+        if 'irmc_firmware_version' in capa_keys:
+            v['irmc_firmware_version'] = \
+                snmp.get_irmc_firmware_version(snmp_client)
+
+        if 'server_model' in capa_keys:
+            v['server_model'] = snmp.get_server_model(snmp_client)
+
+        # Sometime the server started but PCI device list building is
+        # still in progress so system will response error. We have to wait
+        # for some more seconds.
+        if kwargs.get('sleep_flag', False) and \
+                any(k in capa_keys for k in ('pci_gpu_devices', 'cpu_fpga')):
+            time.sleep(5)
+
+        if 'pci_gpu_devices' in capa_keys:
+            v['pci_gpu_devices'] = ipmi.get_pci_device(d_info, gpu_ids)
+
+        if fpga_ids is not None and 'cpu_fpga' in capa_keys:
+            v['cpu_fpga'] = ipmi.get_pci_device(d_info, fpga_ids)
+
+        if 'trusted_boot' in capa_keys:
+            v['trusted_boot'] = ipmi.get_tpm_status(d_info)
+
+        return v
+    except (snmp.SNMPFailure, ipmi.IPMIFailure) as err:
+        raise SCCIClientError('Capabilities inspection failed: %s' % err)
+
+
+def process_session_status(irmc_info, session_timeout, upgrade_type):
+    """process session for Bios config backup/restore or RAID config operation
+
+    :param irmc_info: node info
+    :param session_timeout: session timeout
+    :param upgrade_type: flag to check upgrade with bios or irmc
+    :return: a dict with following values:
+        {
+            'upgrade_message': <Message of firmware upgrade mechanism>,
+            'upgrade_status'
+        }
+    """
+    session_expiration = time.time() + session_timeout
+
+    while time.time() < session_expiration:
+        try:
+            # Get session status to check
+            session = get_firmware_upgrade_status(irmc_info, upgrade_type)
+        except SCCIClientError:
+            # Ignore checking during rebooted server
+            time.sleep(10)
+            continue
+
+        status = session.find("./Value").text
+        severity = session.find("./Severity").text
+        message = session.find("./Message").text
+        result = {}
+
+        if severity == 'Information' and status != '0':
+            if 'FLASH successful' in message:
+                result['upgrade_status'] = 'Complete'
+                return result
+            # Sleep a bit
+            time.sleep(5)
+        elif severity == 'Error':
+            result['upgrade_status'] = 'Error'
+            return result
+        else:
+            # Error occurred, get session log to see what happened
+            session_log = message
+            raise SCCIClientError('Failed to set firmware upgrade. '
+                                  'Session log is %s.' % session_log)
+
+    else:
+        raise SCCISessionTimeout('Failed to time out mechanism with %s.'
+                                 % session_expiration)
+
+
+def get_raid_fgi_status(report):
+    """Gather fgi(foreground initialization) information of raid configuration
+
+    This function returns a fgi status which contains activity status
+    and its values from the report.
+
+    :param report: SCCI report information
+    :returns: dict of fgi status of logical_drives, such as Initializing (10%)
+              or Idle. e.g: {'0': 'Idle', '1': 'Initializing (10%)'}
+    :raises: SCCIInvalidInputError: fail report input.
+             SCCIRAIDNotReady: waiting for RAID configuration to complete.
+    """
+    fgi_status = {}
+    raid_path = "./Software/ServerView/ServerViewRaid"
+
+    if not report.find(raid_path):
+        raise SCCIInvalidInputError(
+            "ServerView RAID not available in Bare metal Server")
+    if not report.find(raid_path + "/amEMSV/System/Adapter/LogicalDrive"):
+        raise SCCIRAIDNotReady(
+            "RAID configuration not configure in Bare metal Server yet")
+
+    logical_drives = report.findall(raid_path +
+                                    "/amEMSV/System/Adapter/LogicalDrive")
+    for logical_drive_name in logical_drives:
+        status = logical_drive_name.find("./Activity").text
+        name = logical_drive_name.find("./LogDriveNumber").text
+        fgi_status.update({name: status})
+
+    return fgi_status
+
+
+def get_firmware_upgrade_status(irmc_info, upgrade_type):
+    """get firmware upgrade status of bios or irmc
+
+    :param irmc_info: dict of iRMC params to access the server node
+        {
+          'irmc_address': host,
+          'irmc_username': user_id,
+          'irmc_password': password,
+          'irmc_port': 80 or 443, default is 443,
+          'irmc_auth_method': 'basic' or 'digest', default is 'digest',
+          'irmc_client_timeout': timeout, default is 60,
+          ...
+        }
+    :param upgrade_type: flag to check upgrade with bios or irmc
+    :raises: ISCCIInvalidInputError if port and/or auth_method params
+             are invalid
+    :raises: SCCIClientError if SCCI failed
+    """
+
+    host = irmc_info.get('irmc_address')
+    userid = irmc_info.get('irmc_username')
+    password = irmc_info.get('irmc_password')
+    port = irmc_info.get('irmc_port', 443)
+    auth_method = irmc_info.get('irmc_auth_method', 'digest')
+    client_timeout = irmc_info.get('irmc_client_timeout', 60)
+
+    auth_obj = None
+    try:
+        protocol = {80: 'http', 443: 'https'}[port]
+        auth_obj = {
+            'basic': requests.auth.HTTPBasicAuth(userid, password),
+            'digest': requests.auth.HTTPDigestAuth(userid, password)
+        }[auth_method.lower()]
+    except KeyError:
+        raise SCCIInvalidInputError(
+            ("Invalid port %(port)d or " +
+             "auth_method for method %(auth_method)s") %
+            {'port': port, 'auth_method': auth_method})
+    try:
+        if upgrade_type == 'bios':
+            upgrade_type = '/biosprogress'
+        elif upgrade_type == 'irmc':
+            upgrade_type = '/irmcprogress'
+        r = requests.get(protocol + '://' + host + upgrade_type,
+                         verify=False,
+                         timeout=(10, client_timeout),
+                         allow_redirects=False,
+                         auth=auth_obj)
+
+        if r.status_code not in (200, 201):
+            raise SCCIClientError(
+                ('HTTP PROTOCOL ERROR, STATUS CODE = %s' %
+                 str(r.status_code)))
+
+        upgrade_status_xml = ET.fromstring(r.text)
+        return upgrade_status_xml
+    except ET.ParseError as parse_error:
+        raise SCCIClientError(parse_error)
